@@ -26,20 +26,23 @@ import Ast
 import Control.Exception (Exception, throwIO)
 import qualified Data.Map.Strict as Map
 import Matcher
-import Pretty (prettyAttribute, prettyBinding, prettyExpression, prettySubst, prettyBytes)
+import Misc (uniqueBindings)
+import Pretty (prettyAttribute, prettyBinding, prettyBytes, prettyExpression, prettySubst)
 import Text.Printf (printf)
 import Yaml (ExtraArgument (..))
 
 data BuildException
-  = CouldNotBuildExpression {_expr :: Expression, _meta :: String}
-  | CouldNotBuildAttribute {_attr :: Attribute, _meta :: String}
-  | CouldNotBuildBinding {_bd :: Binding, _meta :: String}
-  | CouldNotBuildBytes {_bts :: Bytes, _meta :: String}
+  = CouldNotBuildExpression {_expr :: Expression, _msg :: String}
+  | CouldNotBuildAttribute {_attr :: Attribute, _msg :: String}
+  | CouldNotBuildBinding {_bd :: Binding, _msg :: String}
+  | CouldNotBuildBytes {_bts :: Bytes, _msg :: String}
   deriving (Exception)
 
 metaMsg :: String -> String
 metaMsg = printf "meta '%s' is either does not exist or refers to an inappropriate term"
 
+type Built a = Either String a
+
 -- @todo #277:30min Error messages are too verbose. Now, if we can't build expression or binding, we
 --  throw an exception and just print whole expression or binding to console.
 --  If this elements are big, it's just a mess and error message became unreadable. It would be nice to
@@ -48,23 +51,23 @@ instance Show BuildException where
   show CouldNotBuildExpression {..} =
     printf
       "Couldn't build expression, %s\n--Expression: %s"
-      (metaMsg _meta)
+      _msg
       (prettyExpression _expr)
   show CouldNotBuildAttribute {..} =
     printf
       "Couldn't build attribute '%s', %s"
       (prettyAttribute _attr)
-      (metaMsg _meta)
+      _msg
   show CouldNotBuildBinding {..} =
     printf
       "Couldn't build binding, %s\n--Binding: %s"
-      (metaMsg _meta)
+      _msg
       (prettyBinding _bd)
   show CouldNotBuildBytes {..} =
     printf
       "Couldn't build bytes '%s', %s"
       (prettyBytes _bts)
-      (metaMsg _meta)
+      _msg
 
 contextualize :: Expression -> Expression -> Program -> Expression
 contextualize ExGlobal _ (Program expr) = expr
@@ -77,22 +80,22 @@ contextualize (ExApplication expr (BiTau attr bexpr)) context prog =
       bexpr' = contextualize bexpr context prog
    in ExApplication expr' (BiTau attr bexpr')
 
-buildAttribute :: Attribute -> Subst -> Either String Attribute
+buildAttribute :: Attribute -> Subst -> Built Attribute
 buildAttribute (AtMeta meta) (Subst mp) = case Map.lookup meta mp of
   Just (MvAttribute attr) -> Right attr
-  _ -> Left meta
+  _ -> Left (metaMsg meta)
 buildAttribute attr _ = Right attr
 
-buildBytes :: Bytes -> Subst -> Either String Bytes
+buildBytes :: Bytes -> Subst -> Built Bytes
 buildBytes (BtMeta meta) (Subst mp) = case Map.lookup meta mp of
   Just (MvBytes bytes) -> Right bytes
-  _ -> Left meta
+  _ -> Left (metaMsg meta)
 buildBytes bts _ = Right bts
 
 -- Build binding
 -- The function returns [Binding] because the BiMeta is always attached
 -- to the list of bindings
-buildBinding :: Binding -> Subst -> Either String [Binding]
+buildBinding :: Binding -> Subst -> Built [Binding]
 buildBinding (BiTau attr expr) subst = do
   attribute <- buildAttribute attr subst
   (expression, _) <- buildExpression expr subst
@@ -101,18 +104,18 @@ buildBinding (BiVoid attr) subst = do
   attribute <- buildAttribute attr subst
   Right [BiVoid attribute]
 buildBinding (BiMeta meta) (Subst mp) = case Map.lookup meta mp of
-  Just (MvBindings bds) -> Right bds
-  _ -> Left meta
+  Just (MvBindings bds) -> uniqueBindings bds
+  _ -> Left (metaMsg meta)
 buildBinding (BiDelta bytes) subst = do
   bts <- buildBytes bytes subst
   Right [BiDelta bts]
 buildBinding (BiMetaLambda meta) (Subst mp) = case Map.lookup meta mp of
   Just (MvFunction func) -> Right [BiLambda func]
-  _ -> Left meta
+  _ -> Left (metaMsg meta)
 buildBinding binding _ = Right [binding]
 
 -- Build bindings that may contain meta binding (BiMeta)
-buildBindings :: [Binding] -> Subst -> Either String [Binding]
+buildBindings :: [Binding] -> Subst -> Built [Binding]
 buildBindings [] _ = Right []
 buildBindings (bd : rest) subst = do
   first <- buildBinding bd subst
@@ -130,7 +133,7 @@ buildExpressionWithTails expr (tail : rest) subst = case tail of
 -- where X is built expression and Y is context of X
 -- If meta expression is built from MvExpression, is has
 -- context from original Program. It have default context otherwise
-buildExpression :: Expression -> Subst -> Either String (Expression, Expression)
+buildExpression :: Expression -> Subst -> Built (Expression, Expression)
 buildExpression (ExDispatch expr attr) subst = do
   (dispatched, scope) <- buildExpression expr subst
   attr <- buildAttribute attr subst
@@ -140,38 +143,42 @@ buildExpression (ExApplication expr (BiTau battr bexpr)) subst = do
   bds <- buildBinding (BiTau battr bexpr) subst
   Right (ExApplication applied (head bds), scope)
 buildExpression (ExFormation bds) subst = do
-  bds' <- buildBindings bds subst
+  bds' <- buildBindings bds subst >>= uniqueBindings
   Right (ExFormation bds', defaultScope)
 buildExpression (ExMeta meta) (Subst mp) = case Map.lookup meta mp of
-  Just (MvExpression expr scope) -> Right (expr, scope)
-  _ -> Left meta
+  Just (MvExpression expr scope) ->
+    let res = Right (expr, scope)
+     in case expr of
+          ExFormation bds -> uniqueBindings bds >> res
+          _ -> res
+  _ -> Left (metaMsg meta)
 buildExpression (ExMetaTail expr meta) subst = do
   let (Subst mp) = subst
   (expression, scope) <- buildExpression expr subst
   case Map.lookup meta mp of
     Just (MvTail tails) -> Right (buildExpressionWithTails expression tails subst, scope)
-    _ -> Left meta
+    _ -> Left (metaMsg meta)
 buildExpression expr _ = Right (expr, defaultScope)
 
 buildBytesThrows :: Bytes -> Subst -> IO Bytes
 buildBytesThrows bytes subst = case buildBytes bytes subst of
   Right bts -> pure bts
-  Left meta -> throwIO (CouldNotBuildBytes bytes meta)
+  Left msg -> throwIO (CouldNotBuildBytes bytes msg)
 
 buildBindingThrows :: Binding -> Subst -> IO [Binding]
 buildBindingThrows bd subst = case buildBinding bd subst of
   Right bds -> pure bds
-  Left meta -> throwIO (CouldNotBuildBinding bd meta)
+  Left msg -> throwIO (CouldNotBuildBinding bd msg)
 
 buildAttributeThrows :: Attribute -> Subst -> IO Attribute
 buildAttributeThrows attr subst = case buildAttribute attr subst of
   Right attr' -> pure attr'
-  Left meta -> throwIO (CouldNotBuildAttribute attr meta)
+  Left msg -> throwIO (CouldNotBuildAttribute attr msg)
 
 buildExpressionThrows :: Expression -> Subst -> IO (Expression, Expression)
 buildExpressionThrows expr subst = case buildExpression expr subst of
   Right built -> pure built
-  Left meta -> throwIO (CouldNotBuildExpression expr meta)
+  Left msg -> throwIO (CouldNotBuildExpression expr msg)
 
 -- Build a several expression from one expression and several substitutions
 buildExpressions :: Expression -> [Subst] -> IO [(Expression, Expression)]
