@@ -18,7 +18,12 @@ module Misc
     allPathsIn,
     ensuredFile,
     shuffle,
+    toDouble,
     btsToUnescapedStr,
+    attributesFromBindings,
+    uniqueBindings,
+    uniqueBindings',
+    validateYamlObject,
     pattern DataObject,
     pattern DataString,
     pattern DataNumber,
@@ -37,6 +42,7 @@ import Data.ByteString.Lazy (unpack)
 import qualified Data.ByteString.Lazy.UTF8 as U
 import Data.Char (chr, isPrint, ord)
 import Data.List (intercalate)
+import qualified Data.Set as Set
 import qualified Data.Text as T
 import qualified Data.Text.Encoding as T
 import qualified Data.Vector as V
@@ -47,6 +53,9 @@ import System.Directory (doesDirectoryExist, doesFileExist, listDirectory)
 import System.FilePath ((</>))
 import System.Random.Stateful
 import Text.Printf (printf)
+import Data.Aeson (Object)
+import qualified Data.Aeson.KeyMap as KeyMap
+import qualified Data.Aeson.Key as Key
 
 data FsException
   = FileDoesNotExist {file :: FilePath}
@@ -58,8 +67,8 @@ instance Show FsException where
   show DirectoryDoesNotExist {..} = printf "Directory '%s' does not exist" dir
 
 -- Minimal matcher function (required for view pattern)
-matchDataoObject :: Expression -> Maybe (String, Bytes)
-matchDataoObject
+matchDataObject :: Expression -> Maybe (String, Bytes)
+matchDataObject
   ( ExApplication
       (ExDispatch (ExDispatch (ExDispatch ExGlobal (AtLabel "org")) (AtLabel "eolang")) (AtLabel label))
       ( BiTau
@@ -73,7 +82,7 @@ matchDataoObject
             )
         )
     ) = Just (label, bts)
-matchDataoObject _ = Nothing
+matchDataObject _ = Nothing
 
 pattern DataString :: Bytes -> Expression
 pattern DataString bts = DataObject "string" bts
@@ -82,7 +91,7 @@ pattern DataNumber :: Bytes -> Expression
 pattern DataNumber bts = DataObject "number" bts
 
 pattern DataObject :: String -> Bytes -> Expression
-pattern DataObject label bts <- (matchDataoObject -> Just (label, bts))
+pattern DataObject label bts <- (matchDataObject -> Just (label, bts))
   where
     DataObject label bts =
       ExApplication
@@ -98,6 +107,52 @@ pattern DataObject label bts <- (matchDataoObject -> Just (label, bts))
             )
         )
 
+-- Extract attributes from bindings
+attributesFromBindings :: [Binding] -> [Attribute]
+attributesFromBindings [] = []
+attributesFromBindings (bd : bds) =
+  let attr = case bd of
+        BiTau attr _ -> Just attr
+        BiDelta _ -> Just AtDelta
+        BiLambda _ -> Just AtLambda
+        BiVoid attr -> Just attr
+        BiMeta _ -> Nothing
+        BiMetaLambda _ -> Just AtLambda
+   in case attr of
+        Just attr' -> attr' : attributesFromBindings bds
+        _ -> attributesFromBindings bds
+
+uniqueBindings' :: [Binding] -> IO [Binding]
+uniqueBindings' bds = case uniqueBindings bds of
+  Left msg -> throwIO (userError msg)
+  Right _ -> pure bds
+
+-- Check if given binding list consists of unique attributes
+uniqueBindings :: [Binding] -> Either String [Binding]
+uniqueBindings bds = case maybeDuplicatedAttribute bds Set.empty of
+  Just attr ->
+    Left
+      ( printf
+          "Duplicated attribute '%s' found in %s"
+          (show attr)
+          (intercalate ", " (map show (attributesFromBindings bds)))
+      )
+  _ -> Right bds
+  where
+    maybeDuplicatedAttribute :: [Binding] -> Set.Set Attribute -> Maybe Attribute
+    maybeDuplicatedAttribute [] = const Nothing
+    maybeDuplicatedAttribute ((BiTau attr _) : rest) = checkAttr attr rest
+    maybeDuplicatedAttribute (BiVoid attr : rest) = checkAttr attr rest
+    maybeDuplicatedAttribute (BiLambda _ : rest) = checkAttr AtLambda rest
+    maybeDuplicatedAttribute (BiMetaLambda _ : rest) = checkAttr AtLambda rest
+    maybeDuplicatedAttribute (BiDelta _ : rest) = checkAttr AtDelta rest
+    maybeDuplicatedAttribute (BiMeta _ : rest) = maybeDuplicatedAttribute rest
+
+    checkAttr :: Attribute -> [Binding] -> Set.Set Attribute -> Maybe Attribute
+    checkAttr attr rest acc
+      | attr `Set.member` acc = Just attr
+      | otherwise = maybeDuplicatedAttribute rest (Set.insert attr acc)
+
 -- Add void rho binding to the end of the list of any rho binding is not present
 withVoidRho :: [Binding] -> [Binding]
 withVoidRho bds = withVoidRho' bds False
@@ -135,6 +190,11 @@ allPathsIn dir = do
       )
   return (concat paths)
 
+-- >>> toDouble 5
+-- 5.0
+toDouble :: Integer -> Double
+toDouble = fromIntegral
+
 -- >>> btsToWord8 BtEmpty
 -- []
 -- >>> btsToWord8 (BtOne "01")
@@ -289,3 +349,14 @@ shuffle xs = do
     j <- uniformRM (0, i) gen
     M.swap v i j
   V.toList <$> V.freeze v
+
+validateYamlObject :: (Applicative a, MonadFail a) => Object -> [String] -> a ()
+validateYamlObject v keys = do
+  let present = filter (`KeyMap.member` v) (map Key.fromString keys)
+      current = KeyMap.keys v
+  when
+    (length current > 1)
+    (fail ("Exactly one condition type is expected, when multiple condition types specified: " ++ show current))
+  when
+    (null present)
+    (fail (printf "Unknown condition type '%s', expected one of: %s" (show current) (show keys)))
