@@ -17,7 +17,7 @@ import Data.Text (unpack)
 import Data.Yaml (Parser)
 import qualified Data.Yaml as Yaml
 import GHC.Generics
-import Misc (allPathsIn)
+import Misc (allPathsIn, validateYamlObject)
 import Parser
 
 parseJSON' :: String -> (String -> Either String a) -> Value -> Parser a
@@ -52,7 +52,8 @@ instance FromJSON Binding where
 
 instance FromJSON Number where
   parseJSON v = case v of
-    Object o ->
+    Object o -> do
+      validateYamlObject o ["ordinal", "length"]
       asum
         [ Ordinal <$> o .: "ordinal",
           Length <$> o .: "length"
@@ -73,7 +74,8 @@ instance FromJSON Condition where
   parseJSON =
     withObject
       "Condition"
-      ( \v ->
+      ( \v -> do
+          validateYamlObject v ["and", "or", "not", "alpha", "nf", "xi", "eq", "in", "matches", "part-of"]
           asum
             [ And <$> v .: "and",
               Or <$> v .: "or",
@@ -169,6 +171,7 @@ data Extra = Extra
 
 data Rule = Rule
   { name :: Maybe String,
+    description :: Maybe String,
     pattern :: Expression,
     result :: Expression,
     when :: Maybe Condition,
