@@ -10,10 +10,10 @@ import Parser (parseProgramThrows)
 import Pretty
 import Test.Hspec (Example (Arg), Expectation, Spec, SpecWith, describe, it, runIO, shouldBe)
 
-test :: (a -> PrintMode -> String) -> [(String, String, a, PrintMode)] -> SpecWith (Arg Expectation)
+test :: (a -> PrintMode -> Encoding -> String) -> [(String, String, a, PrintMode)] -> SpecWith (Arg Expectation)
 test function useCases =
   forM_ useCases $ \(input, output, arg, mode) ->
-    it input $ function arg mode `shouldBe` output
+    it input $ function arg mode UNICODE `shouldBe` output
 
 prep :: PrintMode -> (String, String) -> IO (String, String, Program, PrintMode)
 prep mode (input, output) = do
@@ -54,7 +54,8 @@ spec = do
             ("Q -> Q.x(~0 -> 1)(~1 -> 2)(~2 -> 3)", "{Φ.x(\n  1,\n  2,\n  3\n)}"),
             ("Q -> Q.x(~0 -> 1)(~2 -> 2)(~1 -> 3)", "{Φ.x(\n  α0 ↦ 1,\n  α2 ↦ 2,\n  α1 ↦ 3\n)}"),
             ("Q -> Φ.jeo.opcode.ldc(18, \"Reading \\\"\")", "{Φ.jeo.opcode.ldc(\n  18,\n  \"Reading \\\"\"\n)}"),
-            ("Q -> [[ k -> [[]] ]]", "{⟦\n  k ↦ ⟦⟧\n⟧}")
+            ("Q -> [[ k -> [[]] ]]", "{⟦\n  k ↦ ⟦⟧\n⟧}"),
+            ("Q -> [[ ]]", "{⟦⟧}")
           ]
     test prettyProgram' useCases
 
